=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2012-01-07 01:45:31 +0000
+++ bzrlib/branch.py	2012-01-18 20:35:41 +0000
@@ -1457,19 +1457,30 @@
         t = transport.get_transport(to_location)
         t.ensure_base()
         format = self._get_checkout_format(lightweight=lightweight)
+        try:
+            checkout = format.initialize_on_transport(t)
+        except errors.AlreadyControlDirError:
+            # It's fine if the control directory already exists,
+            # as long as there is no existing branch and working tree.
+            checkout = controldir.ControlDir.open_from_transport(t)
+            try:
+                checkout.open_branch()
+            except errors.NotBranchError:
+                pass
+            else:
+                raise errors.AlreadyControlDirError(t.base)
+
         if lightweight:
-            checkout = format.initialize_on_transport(t)
-            from_branch = BranchReferenceFormat().initialize(checkout, 
-                target_branch=self)
+            from_branch = checkout.set_branch_reference(target_branch=self)
         else:
-            checkout_branch = controldir.ControlDir.create_branch_convenience(
-                to_location, force_new_tree=False, format=format)
-            checkout = checkout_branch.bzrdir
+            policy = checkout.determine_repository_policy()
+            repo = policy.acquire_repository()[0]
+            checkout_branch = checkout.create_branch()
             checkout_branch.bind(self)
             # pull up to the specified revision_id to set the initial
             # branch tip correctly, and seed it with history.
             checkout_branch.pull(self, stop_revision=revision_id)
-            from_branch=None
+            from_branch = None
         tree = checkout.create_workingtree(revision_id,
                                            from_branch=from_branch,
                                            accelerator_tree=accelerator_tree,

=== modified file 'bzrlib/bzrdir.py'
--- bzrlib/bzrdir.py	2012-01-07 01:06:26 +0000
+++ bzrlib/bzrdir.py	2012-01-18 20:27:41 +0000
@@ -887,6 +887,10 @@
         format = BranchFormatMetadir.find_format(self, name=name)
         return format.get_reference(self, name=name)
 
+    def set_branch_reference(self, target_branch, name=None):
+        format = _mod_branch.BranchReferenceFormat()
+        return format.initialize(self, target_branch=target_branch, name=name)
+
     def get_branch_transport(self, branch_format, name=None):
         """See BzrDir.get_branch_transport()."""
         if name is None:
@@ -1471,10 +1475,13 @@
         # mode from the root directory
         temp_control = lockable_files.LockableFiles(transport,
                             '', lockable_files.TransportLock)
-        temp_control._transport.mkdir('.bzr',
-                                      # FIXME: RBC 20060121 don't peek under
-                                      # the covers
-                                      mode=temp_control._dir_mode)
+        try:
+            temp_control._transport.mkdir('.bzr',
+                # FIXME: RBC 20060121 don't peek under
+                # the covers
+                mode=temp_control._dir_mode)
+        except errors.FileExists:
+            raise errors.AlreadyControlDirError(transport.base)
         if sys.platform == 'win32' and isinstance(transport, local.LocalTransport):
             win32utils.set_file_attr_hidden(transport._abspath('.bzr'))
         file_mode = temp_control._file_mode

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2012-01-19 13:15:39 +0000
+++ bzrlib/commands.py	2012-01-23 15:02:51 +0000
@@ -33,6 +33,7 @@
 
 import bzrlib
 from bzrlib import (
+    config,
     cleanup,
     cmdline,
     debug,

=== modified file 'bzrlib/config.py'
--- bzrlib/config.py	2012-01-13 15:05:42 +0000
+++ bzrlib/config.py	2012-01-20 13:21:01 +0000
@@ -2866,6 +2866,13 @@
     Option('submit_to',
            help='''Where submissions from this branch are mailed to.'''))
 
+option_registry.register_lazy('ssl.ca_certs',
+    'bzrlib.transport.http._urllib2_wrappers', 'opt_ssl_ca_certs')
+
+option_registry.register_lazy('ssl.cert_reqs',
+    'bzrlib.transport.http._urllib2_wrappers', 'opt_ssl_cert_reqs')
+
+
 
 class Section(object):
     """A section defines a dict of option name => value.

=== modified file 'bzrlib/controldir.py'
--- bzrlib/controldir.py	2012-01-06 23:29:07 +0000
+++ bzrlib/controldir.py	2012-01-18 17:46:57 +0000
@@ -230,6 +230,17 @@
             raise errors.NoColocatedBranchSupport(self)
         return None
 
+    def set_branch_reference(self, target_branch, name=None):
+        """Set the referenced URL for the branch in this controldir.
+
+        :param name: Optional colocated branch name
+        :param target_branch: Branch to reference
+        :raises NoColocatedBranchSupport: If a branch name was specified
+            but colocated branches are not supported.
+        :return: The referencing branch
+        """
+        raise NotImplementedError(self.set_branch_reference)
+
     def open_branch(self, name=None, unsupported=False,
                     ignore_fallbacks=False, possible_transports=None):
         """Open the branch object at this ControlDir if one is present.

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2011-12-22 15:33:16 +0000
+++ bzrlib/errors.py	2012-01-20 13:21:01 +0000
@@ -700,6 +700,11 @@
        self.path = urlutils.unescape_for_display(branch.base, 'ascii')
 
 
+class AlreadyControlDirError(PathError):
+
+    _fmt = 'A control directory already exists: "%(path)s".'
+
+
 class AlreadyBranchError(PathError):
 
     _fmt = 'Already a branch: "%(path)s".'
@@ -1667,6 +1672,14 @@
         TransportError.__init__(self, msg, orig_error=orig_error)
 
 
+class CertificateError(TransportError):
+
+    _fmt = "Certificate error: %(error)s"
+
+    def __init__(self, error):
+        self.error = error
+
+
 class InvalidHttpRange(InvalidHttpResponse):
 
     _fmt = "Invalid http range %(range)r for %(path)s: %(msg)s"

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2011-12-18 15:28:38 +0000
+++ bzrlib/info.py	2012-01-18 20:47:13 +0000
@@ -352,7 +352,7 @@
     except (NoWorkingTree, NotLocalUrl, NotBranchError):
         tree = None
         try:
-            branch = a_bzrdir.open_branch()
+            branch = a_bzrdir.open_branch(name="")
         except NotBranchError:
             branch = None
             try:
@@ -442,8 +442,13 @@
             phrase = 'Shared repository'
         else:
             phrase = 'Unshared repository'
+        extra = []
         if repository.make_working_trees():
-            phrase += ' with trees'
+            extra.append('trees')
+        if len(control.get_branches()) > 0:
+            extra.append('colocated branches')
+        if extra:
+            phrase += ' with ' + " and ".join(extra)
         return phrase
     else:
         if repository.is_shared():

=== modified file 'bzrlib/plugins/launchpad/lp_registration.py'
--- bzrlib/plugins/launchpad/lp_registration.py	2011-12-19 10:58:39 +0000
+++ bzrlib/plugins/launchpad/lp_registration.py	2012-01-20 13:07:10 +0000
@@ -56,11 +56,7 @@
 class XMLRPCTransport(xmlrpclib.Transport):
 
     def __init__(self, scheme):
-        # In python2.4 xmlrpclib.Transport is a old-style class, and does not
-        # define __init__, so we check first
-        init = getattr(xmlrpclib.Transport, '__init__', None)
-        if init is not None:
-            init(self)
+        xmlrpclib.Transport.__init__(self)
         self._scheme = scheme
         self._opener = _urllib2_wrappers.Opener()
         self.verbose = 0

=== modified file 'bzrlib/plugins/launchpad/test_lp_directory.py'
--- bzrlib/plugins/launchpad/test_lp_directory.py	2011-10-06 06:45:46 +0000
+++ bzrlib/plugins/launchpad/test_lp_directory.py	2012-01-20 13:07:10 +0000
@@ -19,6 +19,7 @@
 import os
 import xmlrpclib
 
+import bzrlib
 from bzrlib import (
     debug,
     errors,
@@ -29,6 +30,7 @@
 from bzrlib.directory_service import directories
 from bzrlib.tests import (
     features,
+    ssl_certs,
     TestCaseInTempDir,
     TestCaseWithMemoryTransport
 )
@@ -452,12 +454,15 @@
         tests.TestCase.setUp(self)
         self.server = self.server_class()
         self.server.start_server()
+        self.addCleanup(self.server.stop_server)
         # Ensure we don't clobber env
         self.overrideEnv('BZR_LP_XMLRPC_URL', None)
-
-    def tearDown(self):
-        self.server.stop_server()
-        tests.TestCase.tearDown(self)
+        # Ensure we use the right certificates for https.
+        # FIXME: There should be a better way but the only alternative I can
+        # think of involves carrying the ca_certs through the lp_registration
+        # infrastructure to _urllib2_wrappers... -- vila 2012-01-20
+        bzrlib.global_state.cmdline_overrides._from_cmdline(
+            ['ssl.ca_certs=%s' % ssl_certs.build_path('ca.crt')])
 
     def set_canned_response(self, server, path):
         response_format = '''HTTP/1.1 200 OK\r

=== modified file 'bzrlib/plugins/weave_fmt/bzrdir.py'
--- bzrlib/plugins/weave_fmt/bzrdir.py	2012-01-03 13:47:01 +0000
+++ bzrlib/plugins/weave_fmt/bzrdir.py	2012-01-18 19:08:55 +0000
@@ -927,6 +927,12 @@
                                         hardlink=hardlink)
         return result
 
+    def set_branch_reference(self, target_branch, name=None):
+        from bzrlib.branch import BranchReferenceFormat
+        if name is not None:
+            raise errors.NoColocatedBranchSupport(self)
+        raise errors.IncompatibleFormat(BranchReferenceFormat, self._format)
+
 
 class BzrDir4(BzrDirPreSplitOut):
     """A .bzr version 4 control object.

=== modified file 'bzrlib/push.py'
--- bzrlib/push.py	2011-12-19 13:23:58 +0000
+++ bzrlib/push.py	2012-01-19 17:23:00 +0000
@@ -92,11 +92,11 @@
                 revision_id=revision_id, stacked_on=stacked_on,
                 create_prefix=create_prefix, use_existing_dir=use_existing_dir,
                 no_tree=no_tree)
+        except errors.AlreadyControlDirError, err:
+            raise errors.BzrCommandError(gettext(
+                "Target directory %s already contains a .bzr directory, "
+                "but it is not valid.") % (location,))
         except errors.FileExists, err:
-            if err.path.endswith('/.bzr'):
-                raise errors.BzrCommandError(gettext(
-                    "Target directory %s already contains a .bzr directory, "
-                    "but it is not valid.") % (location,))
             if not use_existing_dir:
                 raise errors.BzrCommandError(gettext("Target directory %s"
                      " already exists, but does not have a .bzr"

=== modified file 'bzrlib/reconfigure.py'
--- bzrlib/reconfigure.py	2011-12-19 13:23:58 +0000
+++ bzrlib/reconfigure.py	2012-01-18 17:47:06 +0000
@@ -369,8 +369,7 @@
         else:
             local_branch = self.local_branch
         if self._create_reference:
-            format = branch.BranchReferenceFormat().initialize(self.bzrdir,
-                target_branch=reference_branch)
+            self.bzrdir.set_branch_reference(reference_branch)
         if self._destroy_tree:
             self.bzrdir.destroy_workingtree()
         if self._create_tree:

=== modified file 'bzrlib/remote.py'
--- bzrlib/remote.py	2012-01-07 01:06:26 +0000
+++ bzrlib/remote.py	2012-01-19 14:17:55 +0000
@@ -679,6 +679,29 @@
         b = self.open_branch(name=name)
         return b._format
 
+    def get_branches(self, possible_transports=None, ignore_fallbacks=False):
+        path = self._path_for_remote_call(self._client)
+        try:
+            response, handler = self._call_expecting_body(
+                'BzrDir.get_branches', path)
+        except errors.UnknownSmartMethod:
+            self._ensure_real()
+            return self._real_bzrdir.get_branches()
+        if response[0] != "success":
+            raise errors.UnexpectedSmartServerResponse(response)
+        body = bencode.bdecode(handler.read_body_bytes())
+        ret = {}
+        for (name, value) in body.iteritems():
+            ret[name] = self._open_branch(name, value[0], value[1],
+                possible_transports=possible_transports,
+                ignore_fallbacks=ignore_fallbacks)
+        return ret
+
+    def set_branch_reference(self, target_branch, name=None):
+        """See BzrDir.set_branch_reference()."""
+        self._ensure_real()
+        return self._real_bzrdir.set_branch_reference(target_branch, name=name)
+
     def get_branch_reference(self, name=None):
         """See BzrDir.get_branch_reference()."""
         if name is not None:
@@ -724,25 +747,15 @@
         """See BzrDir._get_tree_branch()."""
         return None, self.open_branch(name=name)
 
-    def open_branch(self, name=None, unsupported=False,
-                    ignore_fallbacks=False, possible_transports=None):
-        if name is None:
-            name = self._get_selected_branch()
-        if unsupported:
-            raise NotImplementedError('unsupported flag support not implemented yet.')
-        if self._next_open_branch_result is not None:
-            # See create_branch for details.
-            result = self._next_open_branch_result
-            self._next_open_branch_result = None
-            return result
-        response = self._get_branch_reference()
-        if response[0] == 'ref':
+    def _open_branch(self, name, kind, location_or_format,
+                     ignore_fallbacks=False, possible_transports=None):
+        if kind == 'ref':
             # a branch reference, use the existing BranchReference logic.
             format = BranchReferenceFormat()
             return format.open(self, name=name, _found=True,
-                location=response[1], ignore_fallbacks=ignore_fallbacks,
+                location=location_or_format, ignore_fallbacks=ignore_fallbacks,
                 possible_transports=possible_transports)
-        branch_format_name = response[1]
+        branch_format_name = location_or_format
         if not branch_format_name:
             branch_format_name = None
         format = RemoteBranchFormat(network_name=branch_format_name)
@@ -750,6 +763,22 @@
             setup_stacking=not ignore_fallbacks, name=name,
             possible_transports=possible_transports)
 
+    def open_branch(self, name=None, unsupported=False,
+                    ignore_fallbacks=False, possible_transports=None):
+        if unsupported:
+            raise NotImplementedError('unsupported flag support not implemented yet.')
+        if self._next_open_branch_result is not None:
+            # See create_branch for details.
+            result = self._next_open_branch_result
+            self._next_open_branch_result = None
+            return result
+        response = self._get_branch_reference()
+        if name is None:
+            name = self._get_selected_branch()
+        return self._open_branch(name, response[0], response[1],
+            possible_transports=possible_transports,
+            ignore_fallbacks=ignore_fallbacks)
+
     def _open_repo_v1(self, path):
         verb = 'BzrDir.find_repository'
         response = self._call(verb, path)

=== modified file 'bzrlib/smart/bzrdir.py'
--- bzrlib/smart/bzrdir.py	2012-01-07 01:06:26 +0000
+++ bzrlib/smart/bzrdir.py	2012-01-19 12:08:42 +0000
@@ -18,7 +18,13 @@
 
 from __future__ import absolute_import
 
-from bzrlib import branch, errors, repository, urlutils
+from bzrlib import (
+    bencode,
+    branch,
+    errors,
+    repository,
+    urlutils,
+    )
 from bzrlib.bzrdir import (
     BzrDir,
     BzrDirFormat,
@@ -425,6 +431,24 @@
         return SuccessfulSmartServerResponse((), content)
 
 
+class SmartServerBzrDirRequestGetBranches(SmartServerRequestBzrDir):
+
+    def do_bzrdir_request(self):
+        """Get the branches in a control directory.
+        
+        The body is a bencoded dictionary, with values similar to the return
+        value of the open branch request.
+        """
+        branches = self._bzrdir.get_branches()
+        ret = {}
+        for name, b in branches.iteritems():
+            if name is None:
+                name = ""
+            ret[name] = ("branch", b._format.network_name())
+        return SuccessfulSmartServerResponse(
+            ("success", ), bencode.bencode(ret))
+
+
 class SmartServerRequestInitializeBzrDir(SmartServerRequest):
 
     def do(self, path):

=== modified file 'bzrlib/smart/request.py'
--- bzrlib/smart/request.py	2011-12-18 12:46:49 +0000
+++ bzrlib/smart/request.py	2012-01-06 22:06:36 +0000
@@ -606,6 +606,9 @@
     'BzrDir.find_repositoryV3', 'bzrlib.smart.bzrdir',
     'SmartServerRequestFindRepositoryV3', info='read')
 request_handlers.register_lazy(
+    'BzrDir.get_branches', 'bzrlib.smart.bzrdir',
+    'SmartServerBzrDirRequestGetBranches', info='read')
+request_handlers.register_lazy(
     'BzrDir.get_config_file', 'bzrlib.smart.bzrdir',
     'SmartServerBzrDirRequestConfigFile', info='read')
 request_handlers.register_lazy(

=== modified file 'bzrlib/switch.py'
--- bzrlib/switch.py	2011-12-18 15:28:38 +0000
+++ bzrlib/switch.py	2012-01-18 19:14:21 +0000
@@ -107,8 +107,17 @@
             b.set_bound_location(to_branch.base)
             b.set_parent(b.get_master_branch().get_parent())
         else:
-            raise errors.BzrCommandError(gettext('Cannot switch a branch, '
-                'only a checkout.'))
+            # If this is a standalone tree and the new branch
+            # is derived from this one, create a lightweight checkout.
+            graph = b.repository.get_graph(to_branch.repository)
+            if (b.bzrdir._format.colocated_branches and
+                 (force or graph.is_ancestor(b.last_revision(),
+                    to_branch.last_revision()))):
+                b.bzrdir.destroy_branch()
+                b.bzrdir.set_branch_reference(to_branch, name="")
+            else:
+                raise errors.BzrCommandError(gettext('Cannot switch a branch, '
+                    'only a checkout.'))
 
 
 def _any_local_commits(this_branch, possible_transports):

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/tests/__init__.py	2011-12-24 10:10:59 +0000
+++ bzrlib/tests/__init__.py	2012-01-04 23:51:16 +0000
@@ -4002,6 +4002,7 @@
         'bzrlib.tests.test_http',
         'bzrlib.tests.test_http_response',
         'bzrlib.tests.test_https_ca_bundle',
+        'bzrlib.tests.test_https_urllib',
         'bzrlib.tests.test_i18n',
         'bzrlib.tests.test_identitymap',
         'bzrlib.tests.test_ignores',

=== modified file 'bzrlib/tests/blackbox/test_branches.py'
--- bzrlib/tests/blackbox/test_branches.py	2011-12-21 14:28:03 +0000
+++ bzrlib/tests/blackbox/test_branches.py	2012-01-18 17:47:06 +0000
@@ -17,7 +17,6 @@
 
 """Black-box tests for bzr branches."""
 
-from bzrlib.branch import BranchReferenceFormat
 from bzrlib.tests import TestCaseWithTransport
 
 
@@ -73,7 +72,7 @@
         t = self.make_repository('a', format='development-colo')
         t.bzrdir.create_branch(name='another')
         branch = t.bzrdir.create_branch(name='colocated')
-        BranchReferenceFormat().initialize(t.bzrdir, target_branch=branch)
+        t.bzrdir.set_branch_reference(target_branch=branch)
         out, err = self.run_bzr('branches a')
         self.assertEquals(out, "  another\n"
                                "* colocated\n")

=== modified file 'bzrlib/tests/blackbox/test_break_lock.py'
--- bzrlib/tests/blackbox/test_break_lock.py	2011-12-14 20:21:52 +0000
+++ bzrlib/tests/blackbox/test_break_lock.py	2012-01-18 17:47:06 +0000
@@ -61,8 +61,7 @@
         local_branch = bzrdir.BzrDir.create_branch_convenience('repo/branch')
         local_branch.bind(self.master_branch)
         checkoutdir = bzrdir.BzrDir.create('checkout')
-        branch.BranchReferenceFormat().initialize(
-            checkoutdir, target_branch=local_branch)
+        checkoutdir.set_branch_reference(local_branch)
         self.wt = checkoutdir.create_workingtree()
 
     def test_break_lock_help(self):

=== modified file 'bzrlib/tests/blackbox/test_checkout.py'
--- bzrlib/tests/blackbox/test_checkout.py	2012-01-05 13:02:31 +0000
+++ bzrlib/tests/blackbox/test_checkout.py	2012-01-18 20:35:41 +0000
@@ -76,6 +76,13 @@
         self.assertEqual(['1'], result.open_workingtree().get_parent_ids())
         self.assertPathDoesNotExist('checkout/added_in_2')
 
+    def test_checkout_into_empty_dir(self):
+        self.make_bzrdir('checkout')
+        out, err = self.run_bzr(['checkout', 'branch', 'checkout'])
+        result = bzrdir.BzrDir.open('checkout')
+        tree = result.open_workingtree()
+        branch = result.open_branch()
+
     def test_checkout_reconstitutes_working_trees(self):
         # doing a 'bzr checkout' in the directory of a branch with no tree
         # or a 'bzr checkout path' with path the name of a directory with

=== modified file 'bzrlib/tests/blackbox/test_info.py'
--- bzrlib/tests/blackbox/test_info.py	2011-12-14 20:21:52 +0000
+++ bzrlib/tests/blackbox/test_info.py	2012-01-19 16:51:39 +0000
@@ -1097,6 +1097,22 @@
        ), out)
         self.assertEqual('', err)
 
+    def test_info_unshared_repository_with_colocated_branches(self):
+        format = bzrdir.format_registry.make_bzrdir('development-colo')
+        transport = self.get_transport()
+
+        # Create unshared repository
+        repo = self.make_repository('repo', shared=False, format=format)
+        repo.set_make_working_trees(True)
+        repo.bzrdir.create_branch(name='foo')
+        out, err = self.run_bzr('info repo')
+        self.assertEqualDiff(
+"""Unshared repository with trees and colocated branches (format: development-colo)
+Location:
+  repository: repo
+""", out)
+        self.assertEqual('', err)
+
     def assertCheckoutStatusOutput(self,
         command_string, lco_tree, shared_repo=None,
         repo_branch=None,
@@ -1256,8 +1272,7 @@
         # Do a light checkout of the heavy one
         transport.mkdir('tree/lightcheckout')
         lco_dir = bzrdir.BzrDirMetaFormat1().initialize('tree/lightcheckout')
-        branch.BranchReferenceFormat().initialize(lco_dir,
-            target_branch=co_branch)
+        lco_dir.set_branch_reference(co_branch)
         lco_dir.create_workingtree()
         lco_tree = lco_dir.open_workingtree()
 

=== modified file 'bzrlib/tests/blackbox/test_shared_repository.py'
--- bzrlib/tests/blackbox/test_shared_repository.py	2011-12-30 14:36:46 +0000
+++ bzrlib/tests/blackbox/test_shared_repository.py	2012-01-19 12:39:30 +0000
@@ -121,7 +121,7 @@
         # being too low. If rpc_count increases, more network roundtrips have
         # become necessary for this use case. Please do not adjust this number
         # upwards without agreement from bzr's network support maintainers.
-        self.assertLength(10, self.hpss_calls)
+        self.assertLength(11, self.hpss_calls)
         self.assertLength(1, self.hpss_connections)
         self.assertThat(self.hpss_calls, ContainsNoVfsCalls)
 

=== modified file 'bzrlib/tests/blackbox/test_status.py'
--- bzrlib/tests/blackbox/test_status.py	2011-11-17 20:11:33 +0000
+++ bzrlib/tests/blackbox/test_status.py	2012-01-18 17:47:06 +0000
@@ -598,8 +598,7 @@
     def make_branch_and_tree(self, relpath):
         source = self.make_branch(pathjoin('..', relpath))
         checkout = bzrdir.BzrDirMetaFormat1().initialize(relpath)
-        bzrlib.branch.BranchReferenceFormat().initialize(checkout,
-            target_branch=source)
+        checkout.set_branch_reference(source)
         return checkout.create_workingtree()
 
 

=== modified file 'bzrlib/tests/blackbox/test_switch.py'
--- bzrlib/tests/blackbox/test_switch.py	2011-12-14 21:57:37 +0000
+++ bzrlib/tests/blackbox/test_switch.py	2012-01-19 12:36:22 +0000
@@ -178,13 +178,38 @@
         self.assertPathExists('checkout/file-1')
         self.assertPathDoesNotExist('checkout/file-2')
 
+    def test_switch_into_colocated(self):
+        # Create a new colocated branch from an existing non-colocated branch.
+        tree = self.make_branch_and_tree('.', format='development-colo')
+        self.build_tree(['file-1', 'file-2'])
+        tree.add('file-1')
+        revid1 = tree.commit('rev1')
+        tree.add('file-2')
+        revid2 = tree.commit('rev2')
+        self.run_bzr(['switch', '-b', 'anotherbranch'])
+        self.assertEquals(
+            ['', 'anotherbranch'],
+            tree.branch.bzrdir.get_branches().keys())
+
+    def test_switch_into_unrelated_colocated(self):
+        # Create a new colocated branch from an existing non-colocated branch.
+        tree = self.make_branch_and_tree('.', format='development-colo')
+        self.build_tree(['file-1', 'file-2'])
+        tree.add('file-1')
+        revid1 = tree.commit('rev1')
+        tree.add('file-2')
+        revid2 = tree.commit('rev2')
+        tree.bzrdir.create_branch(name='foo')
+        self.run_bzr_error(['Cannot switch a branch, only a checkout.'],
+            'switch foo')
+        self.run_bzr(['switch', '--force', 'foo'])
+
     def test_switch_existing_colocated(self):
         # Create a branch branch-1 that initially is a checkout of 'foo'
         # Use switch to change it to 'anotherbranch'
         repo = self.make_repository('branch-1', format='development-colo')
         target_branch = repo.bzrdir.create_branch(name='foo')
-        branch.BranchReferenceFormat().initialize(
-            repo.bzrdir, target_branch=target_branch)
+        repo.bzrdir.set_branch_reference(target_branch)
         tree = repo.bzrdir.create_workingtree()
         self.build_tree(['branch-1/file-1', 'branch-1/file-2'])
         tree.add('file-1')
@@ -203,8 +228,7 @@
         # Use switch to create 'anotherbranch' which derives from that
         repo = self.make_repository('branch-1', format='development-colo')
         target_branch = repo.bzrdir.create_branch(name='foo')
-        branch.BranchReferenceFormat().initialize(
-            repo.bzrdir, target_branch=target_branch)
+        repo.bzrdir.set_branch_reference(target_branch)
         tree = repo.bzrdir.create_workingtree()
         self.build_tree(['branch-1/file-1', 'branch-1/file-2'])
         tree.add('file-1')
@@ -223,8 +247,7 @@
         self.requireFeature(UnicodeFilenameFeature)
         repo = self.make_repository('branch-1', format='development-colo')
         target_branch = repo.bzrdir.create_branch(name='foo')
-        branch.BranchReferenceFormat().initialize(
-            repo.bzrdir, target_branch=target_branch)
+        repo.bzrdir.set_branch_reference(target_branch)
         tree = repo.bzrdir.create_workingtree()
         self.build_tree(['branch-1/file-1', 'branch-1/file-2'])
         tree.add('file-1')

=== modified file 'bzrlib/tests/blackbox/test_update.py'
--- bzrlib/tests/blackbox/test_update.py	2011-09-29 01:03:18 +0000
+++ bzrlib/tests/blackbox/test_update.py	2012-01-18 17:47:06 +0000
@@ -194,8 +194,7 @@
 
         self.build_tree(['checkout1/'])
         checkout_dir = bzrdir.BzrDirMetaFormat1().initialize('checkout1')
-        branch.BranchReferenceFormat().initialize(checkout_dir,
-            target_branch=master.branch)
+        checkout_dir.set_branch_reference(master.branch)
         checkout1 = checkout_dir.create_workingtree('m1')
 
         # Create a second branch, with an extra commit
@@ -251,8 +250,7 @@
 
         self.build_tree(['checkout1/'])
         checkout_dir = bzrdir.BzrDirMetaFormat1().initialize('checkout1')
-        branch.BranchReferenceFormat().initialize(checkout_dir,
-            target_branch=master.branch)
+        checkout_dir.set_branch_reference(master.branch)
         checkout1 = checkout_dir.create_workingtree('m1')
 
         # Create a second branch, with an extra commit

=== modified file 'bzrlib/tests/https_server.py'
--- bzrlib/tests/https_server.py	2011-01-10 22:20:12 +0000
+++ bzrlib/tests/https_server.py	2012-01-19 17:14:27 +0000
@@ -49,7 +49,13 @@
         serving = test_server.TestingTCPServerMixin.verify_request(
             self, request, client_address)
         if serving:
-            request.do_handshake()
+            try:
+                request.do_handshake()
+            except ssl.SSLError, e:
+                # FIXME: We proabaly want more tests to capture which ssl
+                # errors are worth reporting but mostly our tests want an https
+                # server that works -- vila 2012-01-19
+                return False
         return serving
 
     def ignored_exceptions_during_shutdown(self, e):

=== modified file 'bzrlib/tests/matchers.py'
--- bzrlib/tests/matchers.py	2011-12-14 12:15:44 +0000
+++ bzrlib/tests/matchers.py	2012-01-06 22:44:57 +0000
@@ -31,6 +31,7 @@
     'MatchesAncestry',
     'ContainsNoVfsCalls',
     'ReturnsUnlockable',
+    'RevisionHistoryMatches',
     ]
 
 from bzrlib import (
@@ -184,6 +185,31 @@
         return Equals(entries).match(actual)
 
 
+class RevisionHistoryMatches(Matcher):
+    """A matcher that checks if a branch has a specific revision history.
+
+    :ivar history: Revision history, as list of revisions. Oldest first.
+    """
+
+    def __init__(self, history):
+        Matcher.__init__(self)
+        self.expected = history
+
+    def __str__(self):
+        return 'RevisionHistoryMatches(%r)' % self.expected
+
+    def match(self, branch):
+        branch.lock_read()
+        try:
+            graph = branch.repository.get_graph()
+            history = list(graph.iter_lefthand_ancestry(
+                branch.last_revision(), [_mod_revision.NULL_REVISION]))
+            history.reverse()
+        finally:
+            branch.unlock()
+        return Equals(self.expected).match(history)
+
+
 class _NoVfsCallsMismatch(Mismatch):
     """Mismatch describing a list of HPSS calls which includes VFS requests."""
 

=== modified file 'bzrlib/tests/per_bzrdir/test_bzrdir.py'
--- bzrlib/tests/per_bzrdir/test_bzrdir.py	2012-01-07 01:45:31 +0000
+++ bzrlib/tests/per_bzrdir/test_bzrdir.py	2012-01-18 17:47:06 +0000
@@ -376,8 +376,7 @@
         referenced_branch = self.make_branch('referencced')
         dir = self.make_bzrdir('source')
         try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_branch)
+            dir.set_branch_reference(referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
             return
@@ -420,8 +419,7 @@
         referenced_branch = self.make_branch('referencced')
         dir = self.make_bzrdir('source')
         try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_branch)
+            dir.set_branch_reference(referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
             return
@@ -691,7 +689,6 @@
             target_branch = repo.bzrdir.create_branch(name='foo')
         except errors.NoColocatedBranchSupport:
             raise TestNotApplicable('Format does not support colocation')
-        reference = branch.BranchReferenceFormat().initialize(
-            repo.bzrdir, target_branch=target_branch)
+        repo.bzrdir.set_branch_reference(target_branch)
         self.assertEqual(set(["", 'foo']),
                          set(repo.bzrdir.get_branches().keys()))

=== modified file 'bzrlib/tests/per_controldir/test_controldir.py'
--- bzrlib/tests/per_controldir/test_controldir.py	2012-01-06 23:29:07 +0000
+++ bzrlib/tests/per_controldir/test_controldir.py	2012-01-18 20:27:41 +0000
@@ -108,6 +108,17 @@
         self.assertRaises(errors.UninitializableFormat,
             self.bzrdir_format.initialize, t.base)
 
+    def test_multiple_initialization(self):
+        # loopback test to check the current format initializes to itself.
+        if not self.bzrdir_format.is_initializable():
+            # unsupported formats are not loopback testable
+            # because the default open will not open them and
+            # they may not be initializable.
+            raise TestNotApplicable("format is not initializable")
+        self.bzrdir_format.initialize('.')
+        self.assertRaises(errors.AlreadyControlDirError,
+            self.bzrdir_format.initialize, '.')
+
     def test_create_null_workingtree(self):
         dir = self.make_bzrdir('dir1')
         dir.create_repository()
@@ -415,13 +426,56 @@
                 "not support stacking")
         self.assertEqual(child.open_branch().get_stacked_on_url(), branch.base)
 
+    def test_set_branch_reference(self):
+        """set_branch_reference creates a branch reference"""
+        referenced_branch = self.make_branch('referenced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = dir.set_branch_reference(referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            raise TestNotApplicable("control directory does not "
+                "support branch references")
+        self.assertEqual(
+            referenced_branch.bzrdir.root_transport.abspath('') + '/',
+            dir.get_branch_reference())
+
+    def test_set_branch_reference_on_existing_reference(self):
+        """set_branch_reference creates a branch reference"""
+        referenced_branch1 = self.make_branch('old-referenced')
+        referenced_branch2 = self.make_branch('new-referenced')
+        dir = self.make_bzrdir('source')
+        try:
+            reference = dir.set_branch_reference(referenced_branch1)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            raise TestNotApplicable("control directory does not "
+                "support branch references")
+        reference = dir.set_branch_reference(referenced_branch2)
+        self.assertEqual(
+            referenced_branch2.bzrdir.root_transport.abspath('') + '/',
+            dir.get_branch_reference())
+
+    def test_set_branch_reference_on_existing_branch(self):
+        """set_branch_reference creates a branch reference"""
+        referenced_branch = self.make_branch('referenced')
+        dir = self.make_branch('source').bzrdir
+        try:
+            reference = dir.set_branch_reference(referenced_branch)
+        except errors.IncompatibleFormat:
+            # this is ok too, not all formats have to support references.
+            raise TestNotApplicable("control directory does not "
+                "support branch references")
+        self.assertEqual(
+            referenced_branch.bzrdir.root_transport.abspath('') + '/',
+            dir.get_branch_reference())
+
     def test_get_branch_reference_on_reference(self):
         """get_branch_reference should return the right url."""
         referenced_branch = self.make_branch('referenced')
         dir = self.make_bzrdir('source')
         try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_branch)
+            dir.set_branch_reference(referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
             raise TestNotApplicable("control directory does not "
@@ -652,8 +706,7 @@
         referenced_branch = self.make_branch('referenced')
         dir = self.make_bzrdir('source')
         try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_branch)
+            dir.set_branch_reference(referenced_branch)
         except errors.IncompatibleFormat:
             raise TestNotApplicable("format does not support branch "
                 "references")
@@ -672,8 +725,7 @@
         referenced_tree.commit('1', rev_id='1', allow_pointless=True)
         dir = self.make_bzrdir('source')
         try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_tree.branch)
+            dir.set_branch_reference(referenced_tree.branch)
         except errors.IncompatibleFormat:
             raise TestNotApplicable("format does not support branch "
                 "references")
@@ -699,8 +751,7 @@
         referenced_tree.commit('1', rev_id='1', allow_pointless=True)
         dir = self.make_bzrdir('source')
         try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_tree.branch)
+            dir.set_branch_reference(referenced_tree.branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
             raise TestNotApplicable("format does not support "
@@ -851,8 +902,7 @@
         referenced_branch = self.make_branch('referencced')
         dir = self.make_bzrdir('source')
         try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_branch)
+            dir.set_branch_reference(referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
             raise TestNotApplicable("format does not support "
@@ -878,8 +928,7 @@
         referenced_branch = self.make_branch('referencced')
         dir = self.make_bzrdir('source')
         try:
-            reference = bzrlib.branch.BranchReferenceFormat().initialize(dir,
-                target_branch=referenced_branch)
+            dir.set_branch_reference(referenced_branch)
         except errors.IncompatibleFormat:
             # this is ok too, not all formats have to support references.
             raise TestNotApplicable("format does not support "
@@ -1581,8 +1630,7 @@
         master = self.make_branch('branch')
         thisdir = self.make_bzrdir('this')
         try:
-            bzrlib.branch.BranchReferenceFormat().initialize(
-                thisdir, target_branch=master)
+            thisdir.set_branch_reference(master)
         except errors.IncompatibleFormat:
             raise TestNotApplicable("format does not support "
                 "branch references")

=== modified file 'bzrlib/tests/per_controldir_colo/test_supported.py'
--- bzrlib/tests/per_controldir_colo/test_supported.py	2011-12-12 12:08:58 +0000
+++ bzrlib/tests/per_controldir_colo/test_supported.py	2012-01-18 17:37:38 +0000
@@ -134,3 +134,10 @@
         self.assertEqual(['foo'], repo.bzrdir.get_branches().keys())
         self.assertEqual(target_branch.base,
                          repo.bzrdir.get_branches()['foo'].base)
+
+    def test_branch_reference(self):
+        referenced = self.make_branch('referenced')
+        repo = self.make_repository('repo')
+        repo.bzrdir.set_branch_reference(referenced, name='foo')
+        self.assertEquals(referenced.base,
+            repo.bzrdir.get_branch_reference('foo'))

=== modified file 'bzrlib/tests/per_controldir_colo/test_unsupported.py'
--- bzrlib/tests/per_controldir_colo/test_unsupported.py	2012-01-06 23:29:07 +0000
+++ bzrlib/tests/per_controldir_colo/test_unsupported.py	2012-01-18 17:37:38 +0000
@@ -69,6 +69,12 @@
         self.assertRaises(errors.NoColocatedBranchSupport,
             made_control.get_branch_reference, "colo")
 
+    def test_set_branch_reference(self):
+        referenced = self.make_branch('referenced')
+        made_control = self.make_bzrdir_with_repo()
+        self.assertRaises(errors.NoColocatedBranchSupport,
+            made_control.set_branch_reference, referenced, name="colo")
+
     def test_get_branches(self):
         made_control = self.make_bzrdir_with_repo()
         made_control.create_branch()

=== modified file 'bzrlib/tests/per_workingtree/test_workingtree.py'
--- bzrlib/tests/per_workingtree/test_workingtree.py	2012-01-06 19:46:41 +0000
+++ bzrlib/tests/per_workingtree/test_workingtree.py	2012-01-18 17:47:06 +0000
@@ -490,8 +490,7 @@
         # current format
         self.build_tree(['checkout/', 'tree/file'])
         checkout = bzrdir.BzrDirMetaFormat1().initialize('checkout')
-        branch.BranchReferenceFormat().initialize(checkout,
-            target_branch=main_branch)
+        checkout.set_branch_reference(main_branch)
         old_tree = self.workingtree_format.initialize(checkout)
         # now commit to 'tree'
         wt.add('file')
@@ -558,8 +557,7 @@
         # current format
         self.build_tree(['checkout/', 'tree/file'])
         checkout = bzrdir.BzrDirMetaFormat1().initialize('checkout')
-        branch.BranchReferenceFormat().initialize(checkout,
-            target_branch=main_branch)
+        checkout.set_branch_reference(main_branch)
         old_tree = self.workingtree_format.initialize(checkout)
         # now commit to 'tree'
         wt.add('file')

=== modified file 'bzrlib/tests/test_branch.py'
--- bzrlib/tests/test_branch.py	2012-01-04 17:12:42 +0000
+++ bzrlib/tests/test_branch.py	2012-01-18 17:47:06 +0000
@@ -575,7 +575,7 @@
         self.assertEqual(opened_branch.base, target_branch.base)
 
     def test_get_reference(self):
-        """For a BranchReference, get_reference should reutrn the location."""
+        """For a BranchReference, get_reference should return the location."""
         branch = self.make_branch('target')
         checkout = branch.create_checkout('checkout', lightweight=True)
         reference_url = branch.bzrdir.root_transport.abspath('') + '/'

=== modified file 'bzrlib/tests/test_commit_merge.py'
--- bzrlib/tests/test_commit_merge.py	2011-10-05 18:11:19 +0000
+++ bzrlib/tests/test_commit_merge.py	2011-12-13 21:20:35 +0000
@@ -20,7 +20,6 @@
 from bzrlib import (
     check,
     osutils,
-    revision as _mod_revision,
     )
 from bzrlib.errors import PointlessCommit
 from bzrlib.tests import (
@@ -29,6 +28,7 @@
 from bzrlib.tests.features import (
     SymlinkFeature,
     )
+from bzrlib.tests.matchers import RevisionHistoryMatches
 
 
 class TestCommitMerge(TestCaseWithTransport):
@@ -61,10 +61,9 @@
         self.assertEquals(by.revno(), 3)
         graph = wty.branch.repository.get_graph()
         self.addCleanup(wty.lock_read().unlock)
-        self.assertEquals(list(
-            graph.iter_lefthand_ancestry(by.last_revision(),
-                [_mod_revision.NULL_REVISION])),
-            ['y@u-0-2', 'y@u-0-1', base_rev])
+        self.assertThat(by,
+            RevisionHistoryMatches([base_rev, 'y@u-0-1', 'y@u-0-2'])
+            )
         rev = by.repository.get_revision('y@u-0-2')
         self.assertEquals(rev.parent_ids,
                           ['y@u-0-1', 'x@u-0-1'])

=== modified file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	2011-10-12 16:00:13 +0000
+++ bzrlib/tests/test_http.py	2012-01-20 12:56:05 +0000
@@ -32,7 +32,6 @@
 import bzrlib
 from bzrlib import (
     bzrdir,
-    cethread,
     config,
     debug,
     errors,
@@ -128,22 +127,29 @@
         ('urllib,http', dict(_activity_server=ActivityHTTPServer,
                             _transport=_urllib.HttpTransport_urllib,)),
         ]
+    if features.pycurl.available():
+        activity_scenarios.append(
+            ('pycurl,http', dict(_activity_server=ActivityHTTPServer,
+                                _transport=PyCurlTransport,)),)
     if features.HTTPSServerFeature.available():
+        # FIXME: Until we have a better way to handle self-signed certificates
+        # (like allowing them in a test specific authentication.conf for
+        # example), we need some specialized pycurl/urllib transport for tests.
+        # -- vila 2012-01-20
+        from bzrlib.tests import (
+            ssl_certs,
+            )
+        class HTTPS_urllib_transport(_urllib.HttpTransport_urllib):
+
+            def __init__(self, base, _from_transport=None):
+                super(HTTPS_urllib_transport, self).__init__(
+                    base, _from_transport=_from_transport,
+                    ca_certs=ssl_certs.build_path('ca.crt'))
+
         activity_scenarios.append(
             ('urllib,https', dict(_activity_server=ActivityHTTPSServer,
-                                _transport=_urllib.HttpTransport_urllib,)),)
-    if features.pycurl.available():
-        activity_scenarios.append(
-            ('pycurl,http', dict(_activity_server=ActivityHTTPServer,
-                                _transport=PyCurlTransport,)),)
-        if features.HTTPSServerFeature.available():
-            from bzrlib.tests import (
-                ssl_certs,
-                )
-            # FIXME: Until we have a better way to handle self-signed
-            # certificates (like allowing them in a test specific
-            # authentication.conf for example), we need some specialized pycurl
-            # transport for tests.
+                                  _transport=HTTPS_urllib_transport,)),)
+        if features.pycurl.available():
             class HTTPS_pycurl_transport(PyCurlTransport):
 
                 def __init__(self, base, _from_transport=None):
@@ -2120,18 +2126,17 @@
         tests.TestCase.setUp(self)
         self.server = self._activity_server(self._protocol_version)
         self.server.start_server()
+        self.addCleanup(self.server.stop_server)
         _activities = {} # Don't close over self and create a cycle
         def report_activity(t, bytes, direction):
             count = _activities.get(direction, 0)
             count += bytes
             _activities[direction] = count
         self.activities = _activities
-
         # We override at class level because constructors may propagate the
         # bound method and render instance overriding ineffective (an
         # alternative would be to define a specific ui factory instead...)
         self.overrideAttr(self._transport, '_report_activity', report_activity)
-        self.addCleanup(self.server.stop_server)
 
     def get_transport(self):
         t = self._transport(self.server.get_url())

=== added file 'bzrlib/tests/test_https_urllib.py'
--- bzrlib/tests/test_https_urllib.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_https_urllib.py	2012-01-19 15:27:47 +0000
@@ -0,0 +1,109 @@
+# Copyright (C) 2011 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Tests for the SSL support in the urllib HTTP transport.
+
+"""
+
+import os
+import ssl
+
+from bzrlib import (
+    config,
+    trace,
+    )
+from bzrlib.errors import (
+    CertificateError,
+    ConfigOptionValueError,
+    )
+from bzrlib.tests import (
+    TestCase,
+    TestCaseInTempDir,
+    )
+from bzrlib.transport.http import _urllib2_wrappers
+
+
+class CaCertsConfigTests(TestCaseInTempDir):
+
+    def get_stack(self, content):
+        return config.MemoryStack(content.encode('utf-8'))
+
+    def test_default_raises_value_error(self):
+        stack = self.get_stack("")
+        self.overrideAttr(_urllib2_wrappers, "DEFAULT_CA_PATH",
+                "/i-do-not-exist")
+        self.assertRaises(ValueError, stack.get, 'ssl.ca_certs')
+
+    def test_default_exists(self):
+        self.build_tree(['cacerts.pem'])
+        stack = self.get_stack("")
+        path = os.path.join(self.test_dir, "cacerts.pem")
+        self.overrideAttr(_urllib2_wrappers, "DEFAULT_CA_PATH", path)
+        self.assertEquals(path, stack.get('ssl.ca_certs'))
+
+    def test_specified(self):
+        self.build_tree(['cacerts.pem'])
+        path = os.path.join(self.test_dir, "cacerts.pem")
+        stack = self.get_stack("ssl.ca_certs = %s\n" % path)
+        self.assertEquals(path, stack.get('ssl.ca_certs'))
+
+    def test_specified_doesnt_exist(self):
+        path = os.path.join(self.test_dir, "nonexisting.pem")
+        stack = self.get_stack("ssl.ca_certs = %s\n" % path)
+        self.warnings = []
+        def warning(*args):
+            self.warnings.append(args[0] % args[1:])
+        self.overrideAttr(trace, 'warning', warning)
+        self.assertEquals(_urllib2_wrappers.DEFAULT_CA_PATH,
+                          stack.get('ssl.ca_certs'))
+        self.assertLength(1, self.warnings)
+        self.assertContainsRe(self.warnings[0],
+                              "is not valid for \"ssl.ca_certs\"")
+
+
+class CertReqsConfigTests(TestCaseInTempDir):
+
+    def test_default(self):
+        stack = config.MemoryStack("")
+        self.assertEquals(ssl.CERT_REQUIRED, stack.get("ssl.cert_reqs"))
+
+    def test_from_string(self):
+        stack = config.MemoryStack("ssl.cert_reqs = none\n")
+        self.assertEquals(ssl.CERT_NONE, stack.get("ssl.cert_reqs"))
+        stack = config.MemoryStack("ssl.cert_reqs = optional\n")
+        self.assertEquals(ssl.CERT_OPTIONAL, stack.get("ssl.cert_reqs"))
+        stack = config.MemoryStack("ssl.cert_reqs = required\n")
+        self.assertEquals(ssl.CERT_REQUIRED, stack.get("ssl.cert_reqs"))
+        stack = config.MemoryStack("ssl.cert_reqs = invalid\n")
+        self.assertRaises(ConfigOptionValueError, stack.get, "ssl.cert_reqs")
+
+
+class MatchHostnameTests(TestCase):
+
+    def test_no_certificate(self):
+        self.assertRaises(ValueError,
+                          _urllib2_wrappers.match_hostname, {}, "example.com")
+
+    def test_no_valid_attributes(self):
+        self.assertRaises(CertificateError, _urllib2_wrappers.match_hostname,
+                          {"Problem": "Solved"}, "example.com")
+
+    def test_common_name(self):
+        cert = {'subject': ((('commonName', 'example.com'),),)}
+        self.assertIs(None,
+                      _urllib2_wrappers.match_hostname(cert, "example.com"))
+        self.assertRaises(CertificateError, _urllib2_wrappers.match_hostname,
+                          cert, "example.org")

=== modified file 'bzrlib/tests/test_info.py'
--- bzrlib/tests/test_info.py	2011-11-15 17:27:46 +0000
+++ bzrlib/tests/test_info.py	2012-01-19 16:23:53 +0000
@@ -31,49 +31,59 @@
     def test_describe_standalone_layout(self):
         tree = self.make_branch_and_tree('tree')
         self.assertEqual('Empty control directory', info.describe_layout())
-        self.assertEqual('Unshared repository with trees',
-            info.describe_layout(tree.branch.repository))
+        self.assertEqual(
+            'Unshared repository with trees and colocated branches',
+            info.describe_layout(tree.branch.repository, control=tree.bzrdir))
         tree.branch.repository.set_make_working_trees(False)
-        self.assertEqual('Unshared repository',
-            info.describe_layout(tree.branch.repository))
+        self.assertEqual('Unshared repository with colocated branches',
+            info.describe_layout(tree.branch.repository, control=tree.bzrdir))
         self.assertEqual('Standalone branch',
-            info.describe_layout(tree.branch.repository, tree.branch))
+            info.describe_layout(tree.branch.repository, tree.branch,
+                control=tree.bzrdir))
         self.assertEqual('Standalone branchless tree',
-            info.describe_layout(tree.branch.repository, None, tree))
+            info.describe_layout(tree.branch.repository, None, tree,
+                control=tree.bzrdir))
         self.assertEqual('Standalone tree',
-            info.describe_layout(tree.branch.repository, tree.branch, tree))
+            info.describe_layout(tree.branch.repository, tree.branch, tree,
+                control=tree.bzrdir))
         tree.branch.bind(tree.branch)
         self.assertEqual('Bound branch',
-            info.describe_layout(tree.branch.repository, tree.branch))
+            info.describe_layout(tree.branch.repository, tree.branch,
+                control=tree.bzrdir))
         self.assertEqual('Checkout',
-            info.describe_layout(tree.branch.repository, tree.branch, tree))
+            info.describe_layout(tree.branch.repository, tree.branch, tree,
+                control=tree.bzrdir))
         checkout = tree.branch.create_checkout('checkout', lightweight=True)
         self.assertEqual('Lightweight checkout',
             info.describe_layout(checkout.branch.repository, checkout.branch,
-                                 checkout))
+                                 checkout, control=tree.bzrdir))
 
     def test_describe_repository_layout(self):
         repository = self.make_repository('.', shared=True)
         tree = controldir.ControlDir.create_branch_convenience('tree',
             force_new_tree=True).bzrdir.open_workingtree()
-        self.assertEqual('Shared repository with trees',
-            info.describe_layout(tree.branch.repository))
+        self.assertEqual('Shared repository with trees and colocated branches',
+            info.describe_layout(tree.branch.repository, control=tree.bzrdir))
         repository.set_make_working_trees(False)
-        self.assertEqual('Shared repository',
-            info.describe_layout(tree.branch.repository))
+        self.assertEqual('Shared repository with colocated branches',
+            info.describe_layout(tree.branch.repository, control=tree.bzrdir))
         self.assertEqual('Repository branch',
-            info.describe_layout(tree.branch.repository, tree.branch))
+            info.describe_layout(tree.branch.repository, tree.branch,
+                control=tree.bzrdir))
         self.assertEqual('Repository branchless tree',
-            info.describe_layout(tree.branch.repository, None, tree))
+            info.describe_layout(tree.branch.repository, None, tree,
+                control=tree.bzrdir))
         self.assertEqual('Repository tree',
-            info.describe_layout(tree.branch.repository, tree.branch, tree))
+            info.describe_layout(tree.branch.repository, tree.branch, tree,
+                control=tree.bzrdir))
         tree.branch.bind(tree.branch)
         self.assertEqual('Repository checkout',
-            info.describe_layout(tree.branch.repository, tree.branch, tree))
+            info.describe_layout(tree.branch.repository, tree.branch, tree,
+                control=tree.bzrdir))
         checkout = tree.branch.create_checkout('checkout', lightweight=True)
         self.assertEqual('Lightweight checkout',
             info.describe_layout(checkout.branch.repository, checkout.branch,
-                                 checkout))
+                                 checkout, control=tree.bzrdir))
 
     def assertTreeDescription(self, format):
         """Assert a tree's format description matches expectations"""

=== modified file 'bzrlib/tests/test_matchers.py'
--- bzrlib/tests/test_matchers.py	2011-12-14 12:15:44 +0000
+++ bzrlib/tests/test_matchers.py	2012-01-06 22:53:14 +0000
@@ -178,3 +178,27 @@
         self.assertEquals([calls[0].call], mismatch.vfs_calls)
         self.assertEquals("no VFS calls expected, got: append('file')""",
                 mismatch.describe())
+
+
+class TestRevisionHistoryMatches(TestCaseWithTransport):
+
+    def test_empty(self):
+        tree = self.make_branch_and_tree('.')
+        matcher = RevisionHistoryMatches([])
+        self.assertIs(None, matcher.match(tree.branch))
+
+    def test_matches(self):
+        tree = self.make_branch_and_tree('.')
+        tree.commit('msg1', rev_id='a')
+        tree.commit('msg2', rev_id='b')
+        matcher = RevisionHistoryMatches(['a', 'b'])
+        self.assertIs(None, matcher.match(tree.branch))
+
+    def test_mismatch(self):
+        tree = self.make_branch_and_tree('.')
+        tree.commit('msg1', rev_id='a')
+        tree.commit('msg2', rev_id='b')
+        matcher = RevisionHistoryMatches(['a', 'b', 'c'])
+        self.assertEquals(
+            "['a', 'b', 'c'] != ['a', 'b']",
+            matcher.match(tree.branch).describe())

=== modified file 'bzrlib/tests/test_remote.py'
--- bzrlib/tests/test_remote.py	2012-01-02 10:24:02 +0000
+++ bzrlib/tests/test_remote.py	2012-01-06 22:06:36 +0000
@@ -28,12 +28,12 @@
 import zlib
 
 from bzrlib import (
+    bencode,
     branch,
     bzrdir,
     config,
     controldir,
     errors,
-    graph as _mod_graph,
     inventory,
     inventory_delta,
     remote,
@@ -541,6 +541,40 @@
         self.assertFinished(client)
 
 
+class TestBzrDirGetBranches(TestRemote):
+
+    def test_get_branches(self):
+        transport = MemoryTransport()
+        client = FakeClient(transport.base)
+        reference_bzrdir_format = bzrdir.format_registry.get('default')()
+        branch_name = reference_bzrdir_format.get_branch_format().network_name()
+        client.add_success_response_with_body(
+            bencode.bencode({
+                "foo": ("branch", branch_name),
+                "": ("branch", branch_name)}), "success")
+        client.add_success_response(
+            'ok', '', 'no', 'no', 'no',
+                reference_bzrdir_format.repository_format.network_name())
+        client.add_error_response('NotStacked')
+        client.add_success_response(
+            'ok', '', 'no', 'no', 'no',
+                reference_bzrdir_format.repository_format.network_name())
+        client.add_error_response('NotStacked')
+        transport.mkdir('quack')
+        transport = transport.clone('quack')
+        a_bzrdir = RemoteBzrDir(transport, RemoteBzrDirFormat(),
+            _client=client)
+        result = a_bzrdir.get_branches()
+        self.assertEquals(["", "foo"], result.keys())
+        self.assertEqual(
+            [('call_expecting_body', 'BzrDir.get_branches', ('quack/',)),
+             ('call', 'BzrDir.find_repositoryV3', ('quack/', )),
+             ('call', 'Branch.get_stacked_on_url', ('quack/', )),
+             ('call', 'BzrDir.find_repositoryV3', ('quack/', )),
+             ('call', 'Branch.get_stacked_on_url', ('quack/', ))],
+            client._calls)
+
+
 class TestBzrDirDestroyBranch(TestRemote):
 
     def test_destroy_default(self):

=== modified file 'bzrlib/tests/test_smart.py'
--- bzrlib/tests/test_smart.py	2011-12-14 21:24:07 +0000
+++ bzrlib/tests/test_smart.py	2012-01-19 19:18:47 +0000
@@ -28,6 +28,7 @@
 import zlib
 
 from bzrlib import (
+    bencode,
     branch as _mod_branch,
     bzrdir,
     errors,
@@ -458,6 +459,32 @@
         self.assertEqual(expected, request.execute(''))
 
 
+class TestSmartServerBzrDirRequestGetBranches(
+    tests.TestCaseWithMemoryTransport):
+    """Tests for BzrDir.get_branches."""
+
+    def test_simple(self):
+        backing = self.get_transport()
+        branch = self.make_branch('.')
+        request_class = smart_dir.SmartServerBzrDirRequestGetBranches
+        request = request_class(backing)
+        local_result = bencode.bencode(
+            {"": ("branch", branch._format.network_name())})
+        expected = smart_req.SuccessfulSmartServerResponse(
+            ("success", ), local_result)
+        self.assertEqual(expected, request.execute(''))
+
+    def test_empty(self):
+        backing = self.get_transport()
+        dir = self.make_bzrdir('.')
+        request_class = smart_dir.SmartServerBzrDirRequestGetBranches
+        request = request_class(backing)
+        local_result = bencode.bencode({})
+        expected = smart_req.SuccessfulSmartServerResponse(
+            ('success',), local_result)
+        self.assertEqual(expected, request.execute(''))
+
+
 class TestSmartServerRequestInitializeBzrDir(tests.TestCaseWithMemoryTransport):
 
     def test_empty_dir(self):
@@ -485,7 +512,7 @@
         backing = self.get_transport()
         request = smart_dir.SmartServerRequestInitializeBzrDir(backing)
         self.make_bzrdir('subdir')
-        self.assertRaises(errors.FileExists,
+        self.assertRaises(errors.AlreadyControlDirError,
             request.execute, 'subdir')
 
 
@@ -2500,6 +2527,8 @@
             smart_dir.SmartServerBzrDirRequestCheckoutMetaDir)
         self.assertHandlerEqual('BzrDir.cloning_metadir',
             smart_dir.SmartServerBzrDirRequestCloningMetaDir)
+        self.assertHandlerEqual('BzrDir.get_branches',
+            smart_dir.SmartServerBzrDirRequestGetBranches)
         self.assertHandlerEqual('BzrDir.get_config_file',
             smart_dir.SmartServerBzrDirRequestConfigFile)
         self.assertHandlerEqual('BzrDir.open_branch',

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2011-12-24 10:10:59 +0000
+++ bzrlib/transport/__init__.py	2012-01-20 13:15:15 +0000
@@ -1782,7 +1782,7 @@
                  help="Read-only access of branches exported on the web.")
 register_transport_proto('https://',
             help="Read-only access of branches exported on the web using SSL.")
-# The default http implementation is urllib, but https is pycurl if available
+# The default http implementation is urllib, but https uses pycurl if available
 register_lazy_transport('http://', 'bzrlib.transport.http._pycurl',
                         'PyCurlTransport')
 register_lazy_transport('http://', 'bzrlib.transport.http._urllib',

=== modified file 'bzrlib/transport/http/_urllib.py'
--- bzrlib/transport/http/_urllib.py	2011-12-18 15:28:38 +0000
+++ bzrlib/transport/http/_urllib.py	2012-01-20 09:19:14 +0000
@@ -38,14 +38,14 @@
 
     _opener_class = Opener
 
-    def __init__(self, base, _from_transport=None):
+    def __init__(self, base, _from_transport=None, ca_certs=None):
         super(HttpTransport_urllib, self).__init__(
             base, 'urllib', _from_transport=_from_transport)
         if _from_transport is not None:
             self._opener = _from_transport._opener
         else:
             self._opener = self._opener_class(
-                report_activity=self._report_activity)
+                report_activity=self._report_activity, ca_certs=ca_certs)
 
     def _perform(self, request):
         """Send the request to the server and handles common errors.
@@ -175,7 +175,18 @@
         )
     permutations = [(HttpTransport_urllib, http_server.HttpServer_urllib),]
     if features.HTTPSServerFeature.available():
-        from bzrlib.tests import https_server
-        permutations.append((HttpTransport_urllib,
+        from bzrlib.tests import (
+            https_server,
+            ssl_certs,
+            )
+
+        class HTTPS_urllib_transport(HttpTransport_urllib):
+
+            def __init__(self, base, _from_transport=None):
+                super(HTTPS_urllib_transport, self).__init__(
+                    base, _from_transport=_from_transport,
+                    ca_certs=ssl_certs.build_path('ca.crt'))
+
+        permutations.append((HTTPS_urllib_transport,
                              https_server.HTTPSServer_urllib))
     return permutations

=== modified file 'bzrlib/transport/http/_urllib2_wrappers.py'
--- bzrlib/transport/http/_urllib2_wrappers.py	2011-12-19 13:23:58 +0000
+++ bzrlib/transport/http/_urllib2_wrappers.py	2012-01-20 09:19:14 +0000
@@ -14,7 +14,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
-"""Implementaion of urllib2 tailored to bzr needs
+"""Implementation of urllib2 tailored to bzr needs
 
 This file complements the urllib2 class hierarchy with custom classes.
 
@@ -50,6 +50,7 @@
 
 import errno
 import httplib
+import os
 import socket
 import urllib
 import urllib2
@@ -63,12 +64,68 @@
     config,
     debug,
     errors,
+    lazy_import,
     osutils,
     trace,
     transport,
     urlutils,
     )
-
+lazy_import.lazy_import(globals(), """
+import ssl
+""")
+
+DEFAULT_CA_PATH = u"/etc/ssl/certs/ca-certificates.crt"
+
+
+def default_ca_certs():
+    if not os.path.exists(DEFAULT_CA_PATH):
+        raise ValueError("default ca certs path %s does not exist" %
+            DEFAULT_CA_PATH)
+    return DEFAULT_CA_PATH
+
+
+def ca_certs_from_store(path):
+    if not os.path.exists(path):
+        raise ValueError("ca certs path %s does not exist" % path)
+    return path
+
+
+def default_cert_reqs():
+    return u"required"
+
+
+def cert_reqs_from_store(unicode_str):
+    import ssl
+    try:
+        return {
+            "required": ssl.CERT_REQUIRED,
+            "optional": ssl.CERT_OPTIONAL,
+            "none": ssl.CERT_NONE
+            }[unicode_str]
+    except KeyError:
+        raise ValueError("invalid value %s" % unicode_str)
+
+
+opt_ssl_ca_certs = config.Option('ssl.ca_certs',
+        from_unicode=ca_certs_from_store,
+        default=default_ca_certs,
+        invalid='warning',
+        help="""\
+Path to certification authority certificates to trust.
+""")
+
+opt_ssl_cert_reqs = config.Option('ssl.cert_reqs',
+        default=default_cert_reqs,
+        from_unicode=cert_reqs_from_store,
+        invalid='error',
+        help="""\
+Whether to require a certificate from the remote side. (default:required)
+
+Possible values:
+ * none: Certificates ignored
+ * optional: Certificates not required, but validated if provided
+ * required: Certificates required, and validated
+""")
 
 checked_kerberos = False
 kerberos = None
@@ -299,11 +356,12 @@
 
     # XXX: Needs refactoring at the caller level.
     def __init__(self, host, port=None, proxied_host=None,
-                 report_activity=None):
+                 report_activity=None, ca_certs=None):
         AbstractHTTPConnection.__init__(self, report_activity=report_activity)
         # Use strict=True since we don't support HTTP/0.9
         httplib.HTTPConnection.__init__(self, host, port, strict=True)
         self.proxied_host = proxied_host
+        # ca_certs is ignored, it's only relevant for https
 
     def connect(self):
         if 'http' in debug.debug_flags:
@@ -312,29 +370,72 @@
         self._wrap_socket_for_reporting(self.sock)
 
 
-# Build the appropriate socket wrapper for ssl
-try:
-    # python 2.6 introduced a better ssl package
-    import ssl
-    _ssl_wrap_socket = ssl.wrap_socket
-except ImportError:
-    # python versions prior to 2.6 don't have ssl and ssl.wrap_socket instead
-    # they use httplib.FakeSocket
-    def _ssl_wrap_socket(sock, key_file, cert_file):
-        ssl_sock = socket.ssl(sock, key_file, cert_file)
-        return httplib.FakeSocket(sock, ssl_sock)
+# These two methods were imported from Python 3.2's ssl module
+
+def _dnsname_to_pat(dn):
+    pats = []
+    for frag in dn.split(r'.'):
+        if frag == '*':
+            # When '*' is a fragment by itself, it matches a non-empty dotless
+            # fragment.
+            pats.append('[^.]+')
+        else:
+            # Otherwise, '*' matches any dotless fragment.
+            frag = re.escape(frag)
+            pats.append(frag.replace(r'\*', '[^.]*'))
+    return re.compile(r'\A' + r'\.'.join(pats) + r'\Z', re.IGNORECASE)
+
+
+def match_hostname(cert, hostname):
+    """Verify that *cert* (in decoded format as returned by
+    SSLSocket.getpeercert()) matches the *hostname*.  RFC 2818 rules
+    are mostly followed, but IP addresses are not accepted for *hostname*.
+
+    CertificateError is raised on failure. On success, the function
+    returns nothing.
+    """
+    if not cert:
+        raise ValueError("empty or no certificate")
+    dnsnames = []
+    san = cert.get('subjectAltName', ())
+    for key, value in san:
+        if key == 'DNS':
+            if _dnsname_to_pat(value).match(hostname):
+                return
+            dnsnames.append(value)
+    if not san:
+        # The subject is only checked when subjectAltName is empty
+        for sub in cert.get('subject', ()):
+            for key, value in sub:
+                # XXX according to RFC 2818, the most specific Common Name
+                # must be used.
+                if key == 'commonName':
+                    if _dnsname_to_pat(value).match(hostname):
+                        return
+                    dnsnames.append(value)
+    if len(dnsnames) > 1:
+        raise errors.CertificateError(
+            "hostname %r doesn't match either of %s"
+            % (hostname, ', '.join(map(repr, dnsnames))))
+    elif len(dnsnames) == 1:
+        raise errors.CertificateError("hostname %r doesn't match %r" %
+                                      (hostname, dnsnames[0]))
+    else:
+        raise errors.CertificateError("no appropriate commonName or "
+            "subjectAltName fields were found")
 
 
 class HTTPSConnection(AbstractHTTPConnection, httplib.HTTPSConnection):
 
     def __init__(self, host, port=None, key_file=None, cert_file=None,
                  proxied_host=None,
-                 report_activity=None):
+                 report_activity=None, ca_certs=None):
         AbstractHTTPConnection.__init__(self, report_activity=report_activity)
         # Use strict=True since we don't support HTTP/0.9
         httplib.HTTPSConnection.__init__(self, host, port,
                                          key_file, cert_file, strict=True)
         self.proxied_host = proxied_host
+        self.ca_certs = ca_certs
 
     def connect(self):
         if 'http' in debug.debug_flags:
@@ -345,7 +446,38 @@
             self.connect_to_origin()
 
     def connect_to_origin(self):
-        ssl_sock = _ssl_wrap_socket(self.sock, self.key_file, self.cert_file)
+        # FIXME JRV 2011-12-18: Use location config here?
+        config_stack = config.GlobalStack()
+        if self.ca_certs is None:
+            ca_certs = config_stack.get('ssl.ca_certs')
+        else:
+            ca_certs = self.ca_certs
+        cert_reqs = config_stack.get('ssl.cert_reqs')
+        if cert_reqs == ssl.CERT_NONE:
+            trace.warning("not checking SSL certificates for %s: %d",
+                self.host, self.port)
+        else:
+            if ca_certs is None:
+                trace.warning(
+                    "no valid trusted SSL CA certificates file set. See "
+                    "'bzr help ssl.ca_certs' for more information on setting "
+                    "trusted CA's.")
+        try:
+            ssl_sock = ssl.wrap_socket(self.sock, self.key_file, self.cert_file,
+                cert_reqs=cert_reqs, ca_certs=ca_certs)
+        except ssl.SSLError, e:
+            if e.errno != ssl.SSL_ERROR_SSL:
+                raise
+            trace.note(
+                "To disable SSL certificate verification, use "
+                "-Ossl.cert_reqs=none. See ``bzr help ssl.ca_certs`` for "
+                "more information on specifying trusted CA certificates.")
+            raise
+        peer_cert = ssl_sock.getpeercert()
+        if (cert_reqs == ssl.CERT_REQUIRED or
+            (cert_reqs == ssl.CERT_OPTIONAL and peer_cert)):
+            match_hostname(peer_cert, self.host)
+
         # Wrap the ssl socket before anybody use it
         self._wrap_socket_for_reporting(ssl_sock)
 
@@ -453,8 +585,9 @@
 
     handler_order = 1000 # after all pre-processings
 
-    def __init__(self, report_activity=None):
+    def __init__(self, report_activity=None, ca_certs=None):
         self._report_activity = report_activity
+        self.ca_certs = ca_certs
 
     def create_connection(self, request, http_connection_class):
         host = request.get_host()
@@ -468,7 +601,8 @@
         try:
             connection = http_connection_class(
                 host, proxied_host=request.proxied_host,
-                report_activity=self._report_activity)
+                report_activity=self._report_activity,
+                ca_certs=self.ca_certs)
         except httplib.InvalidURL, exception:
             # There is only one occurrence of InvalidURL in httplib
             raise errors.InvalidURL(request.get_full_url(),
@@ -660,6 +794,10 @@
                     % (request, request.connection.sock.getsockname())
             response = connection.getresponse()
             convert_to_addinfourl = True
+        except (ssl.SSLError, errors.CertificateError):
+            # Something is wrong with either the certificate or the hostname,
+            # re-trying won't help
+            raise
         except (socket.gaierror, httplib.BadStatusLine, httplib.UnknownProtocol,
                 socket.error, httplib.HTTPException):
             response = self.retry_or_raise(http_class, request, first_try)
@@ -1657,9 +1795,10 @@
                  connection=ConnectionHandler,
                  redirect=HTTPRedirectHandler,
                  error=HTTPErrorProcessor,
-                 report_activity=None):
+                 report_activity=None,
+                 ca_certs=None):
         self._opener = urllib2.build_opener(
-            connection(report_activity=report_activity),
+            connection(report_activity=report_activity, ca_certs=ca_certs),
             redirect, error,
             ProxyHandler(),
             HTTPBasicAuthHandler(),

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2012-01-03 13:47:01 +0000
+++ bzrlib/workingtree.py	2012-01-18 17:47:06 +0000
@@ -2420,8 +2420,7 @@
         tree_transport = self.bzrdir.root_transport.clone(sub_path)
         if tree_transport.base != branch_transport.base:
             tree_bzrdir = format.initialize_on_transport(tree_transport)
-            branch.BranchReferenceFormat().initialize(tree_bzrdir,
-                target_branch=new_branch)
+            tree_bzrdir.set_branch_reference(new_branch)
         else:
             tree_bzrdir = branch_bzrdir
         wt = tree_bzrdir.create_workingtree(_mod_revision.NULL_REVISION)

=== modified file 'doc/en/release-notes/bzr-2.5.txt'
--- doc/en/release-notes/bzr-2.5.txt	2012-01-19 13:15:39 +0000
+++ doc/en/release-notes/bzr-2.5.txt	2012-01-23 15:02:51 +0000
@@ -26,6 +26,18 @@
 .. Improvements to existing commands, especially improved performance 
    or memory usage, or better results.
 
+* "bzr switch -b" in a standalone tree will now create a colocated branch.
+  (Jelmer Vernooij, #918197)
+
+* ``bzr info`` now reports when there are present (but unused) colocated
+  branches. (Jelmer Vernooij, #891646)
+
+* Checkouts can now be into target directories that already have
+  a control directory (but no branch or working tree).
+  (Jelmer Vernooij, #913980)
+
+* New HPSS call for ``BzrDir.get_branches``. (Jelmer Vernooij, #894460)
+
 * Two new command hooks, ``pre_command`` and ``post_command``,
   provide notification before and after a command has been run.
   (Brian de Alwis, Jelmer Vernooij)
@@ -45,6 +57,10 @@
 * Test for equality instead of object identity where ROOT_PARENT is concerned.
   (Wouter van Heyst, #881142)
 
+* urllib-based HTTPS client connections now verify the server certificate
+  validity as well as the hostname.
+  (Jelmer Vernooij, Vincent Ladeuil, #651161)
+
 Documentation
 *************
 
@@ -56,12 +72,17 @@
 .. Changes that may require updates in plugins or other code that uses
    bzrlib.
 
+* ``ControlDir`` now has a new method ``set_branch_reference`` which can
+  be used for setting branch references. (Jelmer Vernooij)
+
 Internals
 *********
 
 .. Major internal changes, unlikely to be visible to users or plugin 
    developers, but interesting for bzr developers.
 
+* A new matcher ``RevisionHistoryMatches`` has been added. (Jelmer Vernooij)
+
 * ``MutableTree`` has two new hooks ``pre_transform`` and
   ``post_transform`` that are called for tree transform operations.
   (Jelmer Vernooij, #912084)
@@ -79,7 +100,7 @@
 
 :2.5b5: 2012-01-12
 
-This is the firth (ans last) beta of the 2.5 series, leading to a 2.5.0
+This is the fifth (and last) beta of the 2.5 series, leading to a 2.5.0
 release in February 2012. Beta releases are suitable for everyday use but
 may cause some incompatibilities with plugins.
 

=== modified file 'doc/en/whats-new/whats-new-in-2.5.txt'
--- doc/en/whats-new/whats-new-in-2.5.txt	2012-01-03 16:28:43 +0000
+++ doc/en/whats-new/whats-new-in-2.5.txt	2012-01-04 23:51:16 +0000
@@ -36,6 +36,23 @@
 encoding for interacting with the filesystem. This makes creating a working
 tree and commiting to it possible for such branches in most environments.
 
+SSL Certificate Verification Support in urllib HTTPS backend
+************************************************************
+
+In previous versions of Bazaar, only one of the two supported HTTPS
+backends, pycurl, supported verification of SSL certificates. This version
+also introduces this support for the urllib backend.
+
+Along with this support two new options have been introduced to control
+which CA's are trusted and to what degree server certificates should be
+verified. See ``bzr help ssl.ca_certs`` and ``bzr help ssl.cert_reqs``
+for more information
+
+Users who have previously used the urllib HTTPS backend with servers
+with invalid or untrusted certificates can continue to do so by
+adding the required certificates to the trusted CA certificate file
+(recommended) or by setting the ``ssl.cert_reqs`` option to ``none``.
+
 Faster smart server
 *******************
 

